/*++

Copyright (c) Sydus.

Module:

    DataWriter.h

Authors:

    Sharvil Nanavati (sharvil) 2006-12-03

--*/

#ifndef DATAWRITER_H
#define DATAWRITER_H

#include "base.h"
#include "ByteOrder.h"
#include "StringEncoding.h"

template <typename ByteOrder = BigEndian, typename StringEncoding = Length8Ascii>
class DataWriter
{
	public:
		DataWriter();
		~DataWriter();

		void AppendInt8(int8 value);
		void AppendInt16(int16 value);
		void AppendInt32(int32 value);

		void AppendUint8(uint8 value);
		void AppendUint16(uint16 value);
		void AppendUint32(uint32 value);

		void AppendString(const std::string & str, std::string::size_type limit = std::string::npos);
		void AppendString(const BString & str);

		void WriteInt8(BString::size_type offset, int8 value);
		void WriteInt16(BString::size_type offset, int16 value);
		void WriteInt32(BString::size_type offset, int32 value);

		void WriteUint8(BString::size_type offset, uint8 value);
		void WriteUint16(BString::size_type offset, uint16 value);
		void WriteUint32(BString::size_type offset, uint32 value);

		void clear();
		void Zero();

		BString::size_type Length() const;
		BString AsBString() const;

	private:
		BString iData;
};

template <typename T1, typename T2>
DataWriter <T1, T2>::DataWriter()
{
}

template <typename T1, typename T2>
DataWriter <T1, T2>::~DataWriter()
{
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::AppendInt8(int8 value)
{
	uint32 length = iData.GetLength();

	iData.Resize(length + 1);
	iData[length] = (uint8)value;
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::AppendInt16(int16 value)
{
	uint32 length = iData.GetLength();

	iData.Resize(length + 2);
	T1::SetInt16(iData, length, value);
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::AppendInt32(int32 value)
{
	uint32 length = iData.GetLength();

	iData.Resize(length + 4);
	T1::SetInt32(iData, length, value);
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::AppendUint8(uint8 value)
{
	uint32 length = iData.GetLength();

	iData.Resize(length + 1);
	iData[length] = value;
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::AppendUint16(uint16 value)
{
	uint32 length = iData.GetLength();

	iData.Resize(length + 2);
	T1::SetUint16(iData, length, value);
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::AppendUint32(uint32 value)
{
	uint32 length = iData.GetLength();

	iData.Resize(length + 4);
	T1::SetUint32(iData, length, value);
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::AppendString(const std::string & str, std::string::size_type limit)
{
	if(str.length() > limit)
		iData.Append(T2::SerializeString(str.substr(0, limit)));
	else
		iData.Append(T2::SerializeString(str));
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::AppendString(const BString & str)
{
	iData.Append(str);
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::WriteInt8(BString::size_type offset, int8 value)
{
	iData[offset] = value;
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::WriteInt16(BString::size_type offset, int16 value)
{
	T1::SetInt16(iData, offset, value);
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::WriteInt32(BString::size_type offset, int32 value)
{
	T1::SetInt32(iData, offset, value);
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::WriteUint8(BString::size_type offset, uint8 value)
{
	T1::SetUint8(iData, offset, value);
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::WriteUint16(BString::size_type offset, uint16 value)
{
	T1::SetUint16(iData, offset, value);
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::WriteUint32(BString::size_type offset, uint32 value)
{
	T1::SetUint32(iData, offset, value);
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::clear()
{
	iData.clear();
}

template <typename T1, typename T2>
void DataWriter <T1, T2>::Zero()
{
	iData.Zero();
}

template <typename T1, typename T2>
BString::size_type DataWriter <T1, T2>::Length() const
{
	return iData.GetLength();
}

template <typename T1, typename T2>
BString DataWriter <T1, T2>::AsBString() const
{
	return iData;
}

#endif
