/*++
 
Copyright (c) Sydus.
 
Module:
 
	Handle.h
 
Authors:
 
	Serge Zloto (szloto) 2006-11-06
 
--*/

#pragma once

#include "base.h"

/**
 * An extrinsic handle, i.e a smart pointer with reference 
 * count maintained within the pointer, not the object pointed to.
 * Once an object is handled, its raw pointer must no longer be used.
 * Handles may not be instantiated on the heap, in order to prevent leaks.
 */
template <typename T> 
class Handle : public NonPointer
{
public:
	Handle(T * object = 0):iObject(object), iRefCount(new uint32(1))
	{
	}

	Handle(const Handle<T>& h):iObject(h.iObject), iRefCount(h.iRefCount)
	{
		++(*iRefCount);
	}

	~Handle()
	{
		decrementRefCount();
	}
	
	Handle<T> &operator=(T* object)
	{
		decrementRefCount();
		iObject = object;
		iRefCount = new uint32(1);
		return *this;
	}

	Handle<T> &operator=(const Handle<T>& h)
	{
		if (iRefCount != h.iRefCount)
		{
			decrementRefCount();

			iObject = h.iObject;
			iRefCount = h.iRefCount;
			++(*iRefCount);
		}
		return *this;
	}

	T *operator->()
	{
		if (iObject == 0)
		{
			throw null_pointer("Null pointer dereferenced from Handle");
		}
		return iObject;
	}

	const T *operator->() const
	{
		if (iObject == 0)
		{
			throw null_pointer("Null pointer dereferenced from Handle");
		}
		return iObject;
	}

	T & operator*()
	{
		if (iObject == 0)
		{
			throw null_pointer("Null pointer dereferenced from Handle");
		}
		return *iObject;
	}

	const T & operator*() const
	{
		if (iObject == 0)
		{
			throw null_pointer("Null pointer dereferenced from Handle");
		}
		return *iObject;
	}

	bool operator==(const Handle<T>& other) const
	{
		return iObject == other.iObject;
	}

	bool operator!=(const Handle<T>& other) const
	{
		return !(*this == other);
	}

	bool isNull() const
	{
		return iObject == 0;
	}

	uint32 getRefCount() const
	{
		return *iRefCount;
	}

private:

	void decrementRefCount()
	{
		if (--(*iRefCount) == 0)
		{
			// This handle is the last owner
			// Delete the ref count first since deleting the object
			// might throw
			delete iRefCount;
			delete iObject;
			iRefCount = 0;
			iObject = 0;
		}
	}

	T *iObject;
	uint32 *iRefCount;
};
